package fr.eurogiciel.ora.gps;

import java.io.IOException;
import java.util.UUID;

import android.app.Service;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.content.Context;
import android.content.Intent;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.os.IBinder;
import android.util.Log;
import fr.eurogiciel.ora.bluetooth.BTClient;
import fr.eurogiciel.ora.bluetooth.BTMessage;
import fr.eurogiciel.ora.bluetooth.BTSocketListener;
import fr.eurogiciel.ora.bluetooth.gps.GpsMsgFactory;

public class GpsService extends Service implements BTSocketListener.Callback{
	private final String TAG = getClass().getPackage().getName();
	
	// client
	public final static String MSG_BT_UUID = "MSG_BT_UUID";
	public final static String MSG_MAC_BT_DEVICE_ADDRESS = "MSG_MAC_BT_DEVICE_ADDRESS";
	private BluetoothDevice device;
	private BTClient client;
	
	
	// GPS
	private MyLocationListener locationListener;
	private static final long GPS_MIN_TIME = 250; //minimum time interval between location updates, in milliseconds 
	private static final float GPS_MIN_DISTANCE = 0.0f;	//minimum distance between location updates, in meters
	private GpsMsgFactory	gpsMsgFactory	= new GpsMsgFactory();
	private boolean enabledGPS;
	
	@Override
	public int onStartCommand(Intent intent, int flags, int startId) {
		final String macBTDeviceAddress = intent.getStringExtra(MSG_MAC_BT_DEVICE_ADDRESS);
		final UUID serv_UUID = UUID.fromString(intent.getStringExtra(MSG_BT_UUID));

		enabledGPS = enableLocationMsgs();
		
		
		
		BluetoothAdapter ba = BluetoothAdapter.getDefaultAdapter();
		
		//retrieve the bt device
		for (BluetoothDevice d : ba.getBondedDevices()) {
			if (d.getAddress().equals(macBTDeviceAddress)) {
				device = d;
				break;
			}
		}
		
		if (device == null) {
			throw new IllegalArgumentException();
		}
		
		
		//Connection thread
		new Thread("Client creation") {
			public void run() {
				try {
					client = new BTClient(GpsService.this, GpsService.this, device, serv_UUID);
				} catch (IOException e) {
					
					Log.e(TAG, e.getMessage(), e);
					stopSelf();
				}
			};
		}.start();
		
		return 0;
	  }
	  
	private boolean enableLocationMsgs() {
		
		try {
			
			LocationManager locationManager = (LocationManager)getSystemService(Context.LOCATION_SERVICE);	
			locationListener = new MyLocationListener();
			locationManager.requestLocationUpdates(LocationManager.GPS_PROVIDER, GPS_MIN_TIME, GPS_MIN_DISTANCE, locationListener);
			
			return true;
		} 
		
		//GPS feature is not available
		catch (SecurityException e) {} 
		catch (NullPointerException e) {}
		catch (IllegalArgumentException e) {}
		return false;
	}
	
	  private class MyLocationListener implements LocationListener 
		{
			@Override
			public void onLocationChanged(Location loc) {
		
				if (client != null && loc != null)
				{
					try {
						client.sendMessage(gpsMsgFactory.createGpsMsg(loc));
					} catch (IOException e) {
						
						Log.e(TAG, "error", e);
					}
				}
			}
		
			@Override
			public void onProviderDisabled(String provider) {
				
			}
		
			@Override
			public void onProviderEnabled(String provider) {
				
			}
		
			@Override
			public void onStatusChanged(String provider, int status, Bundle extras) {}
		}
		
	  private void stopGPSMsgs() {
			if (enabledGPS) {
				LocationManager locationManager = (LocationManager)getSystemService(Context.LOCATION_SERVICE);
				locationManager.removeUpdates(locationListener);
				locationListener = null;
				enabledGPS = false;
			}
		}
	@Override
	public IBinder onBind(Intent arg0) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public void onDestroy()
	{		
		stopGPSMsgs();
		if (client != null) {
			client.close();
		}
		
		super.onDestroy();
	}
	@Override
	public BTMessage onMsgReceived(BluetoothDevice sender, BTMessage msg) {
		//nothing to do, one way communication
		return null;
	}
	@Override
	public boolean isAuthorized(BluetoothDevice device) {
		return true;
	}
	@Override
	public void onDisconnected(BluetoothDevice device) {
		stopGPSMsgs();
		
		stopSelf();
		
	}
}
