package fr.eurogiciel.ora.gps;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.UUID;

import fr.eurogiciel.ora.gps.R;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.Fragment;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.os.ParcelUuid;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.ArrayAdapter;
import android.widget.ListView;

public class LoginActivity extends Activity {
	
	private static final String TAG = LoginActivity.class.getName();
	private static final int REQUEST_ENABLE_BT = 1;
	
	private static final UUID serv_UUID = UUID.fromString("00001101-0000-1000-8000-00805F9B34AC");
	List<BluetoothDevice> validDevices = new ArrayList<BluetoothDevice>();
	
	ListView devListView;
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_login);

		if (savedInstanceState == null) {
			getFragmentManager().beginTransaction()
					.add(R.id.container, new PlaceholderFragment()).commit();
		}
	}

	@Override
	protected void onStart() {
		super.onStart();
		devListView = (ListView)findViewById(R.id.server_listview);
		
		//Refresh the listview of MAC addresses and names.
		RefreshDeviceList();
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {

		// Inflate the menu; this adds items to the action bar if it is present.
		getMenuInflater().inflate(R.menu.login, menu);
		return true;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		// Handle action bar item clicks here. The action bar will
		// automatically handle clicks on the Home/Up button, so long
		// as you specify a parent activity in AndroidManifest.xml.
		int id = item.getItemId();
		if (id == R.id.action_settings) {
			return true;
		}
		return super.onOptionsItemSelected(item);
	}

	/**
	 * A placeholder fragment containing a simple view.
	 */
	public static class PlaceholderFragment extends Fragment {

		public PlaceholderFragment() {
		}

		@Override
		public View onCreateView(LayoutInflater inflater, ViewGroup container,
				Bundle savedInstanceState) {
			View rootView = inflater.inflate(R.layout.fragment_login,
					container, false);
			return rootView;
		}
	}
	
	/**
	 * Callback for the refresh button.
	 * @param v
	 */
	public void onBtnClicked(View v){
	    if(v.getId() == R.id.btn_refresh){
	        RefreshDeviceList();
	    }
	}
	
	/**
	 * Refresh the list of paired devices. Note that a device should be paired to be listed
	 * in the listview on this GUI.
	 */
	private void RefreshDeviceList() {
		BluetoothAdapter mBluetoothAdapter = BluetoothAdapter.getDefaultAdapter();
		
		//Flush the list of valid devices
		validDevices.clear();
		
		if (mBluetoothAdapter == null) 
			Log.e("BTDeviceDiscover", "No bluetooth adapter detected/available.");
		
		//Get the list of paired devices.
		Set<BluetoothDevice> devList = mBluetoothAdapter.getBondedDevices();
		
		//Check if bluetooth is enabled.
		if (!mBluetoothAdapter.isEnabled()) {
		    Intent enableBtIntent = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
		    startActivityForResult(enableBtIntent, REQUEST_ENABLE_BT);
		    
		    Log.e("BTDeviceDiscover", "Bluetooth is DISABLED. Asking the user");
		    return;
		}
		
		for (BluetoothDevice btDevice : devList) {
			//Refresh the list of UUID on the remote device. !!!!ASYNCHRONE
			//btDevice.fetchUuidsWithSdp();
			
			//Get the list of available UUID on the remote device.
			ParcelUuid[] puList = btDevice.getUuids();
			
			//Search our UUID server through the list of available UUID on the remote device
			for (ParcelUuid pu : puList) {
				if (pu.getUuid().compareTo(serv_UUID) == 0)
				{
					validDevices.add(btDevice); //UUID detected!
					break;
				}
			}
		}
		
		if (validDevices.size() == 0) {
			Log.e(TAG, "No compatible devices found.");
			devListView.setClickable(false);
			String[] noDeviceFound = {"No device found. Please bound your device with ORA Glasses. "};
			devListView.setAdapter(new ArrayAdapter<String>(this,  
					android.R.layout.simple_list_item_1, noDeviceFound));
		}
		else
		{
			//Refresh the list of devices on the GUI.
			BTListAdapter adapter = new BTListAdapter(this, validDevices);
			devListView.setAdapter(adapter);
			devListView.setClickable(true);
			
			OnItemClickListener handler = new OnItemClickListener() {
				@Override
				public void onItemClick(AdapterView<?> parent, View view,
						int position, long id) {
					
					BluetoothDevice btDevice = validDevices.get(position);
					
					//Start the main activity
		    		Intent intent = new Intent(LoginActivity.this, MainActivity.class);
		    	    intent.putExtra(MainActivity.MSG_BT_UUID, LoginActivity.serv_UUID.toString());
		    	    intent.putExtra(MainActivity.MSG_MAC_BT_DEVICE_ADDRESS, btDevice.getAddress());
		    	    
		    		startActivity(intent);
				}
				
			};
			
			devListView.setOnItemClickListener(handler);
		}
	}
	
	/**
	 * Callback for the dialog that ask the user to activate the bluetooth.
	 * This method will automatically close the application if the user didn't activated Bluetooth.
	 * @param requestCode Code that identify to which code the callback is returning. Should be equal to
	 * REQUEST_ENABLE_BT.
	 * @param resultCode Result of the user action.
	 * @param data Not used.
	 */
	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {

		//Check the result of the Bluetooth activation request.
	    if (requestCode == REQUEST_ENABLE_BT) {
	        if(resultCode == RESULT_OK){
	            RefreshDeviceList();
	        }
	        if (resultCode == RESULT_CANCELED) {
	        	//If the BLuetooth is till not activated, inform the user that the application will be closed.
	            AlertDialog.Builder builder = new AlertDialog.Builder(this);
	            builder.setTitle("Application will close...")
	            	   .setMessage("This application can't work without Bluetooth.")
	            	   .setCancelable(false)
	            	   .setPositiveButton("OK", new DialogInterface.OnClickListener() {
						
						@Override
						public void onClick(DialogInterface dialog, int which) {
							finish();
						}
					});
	            AlertDialog dialog = builder.create();
	            dialog.show();
	        }
	    }
	}
}
