package fr.eurogiciel.ora.gps;

import java.util.UUID;

import fr.eurogiciel.ora.gps.R;

import android.app.Activity;
import android.bluetooth.BluetoothAdapter;
import android.content.Context;
import android.content.Intent;
import android.content.res.Configuration;
import android.location.LocationManager;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.Toast;

public class MainActivity extends Activity {
	
	private static final int TOAST_DURATION = 2000;
	private static final int REQUEST_ENABLE_BT = 1;
	
	public final static String MSG_BT_UUID = "MSG_BT_UUID";
	public final static String MSG_MAC_BT_DEVICE_ADDRESS = "MSG_MAC_BT_DEVICE_ADDRESS";
	private String macBTDeviceAddress;
	private static UUID serv_UUID;
	private Button buttonGps = null;
	Intent intent = null;
	private boolean startGPS = false;
	

	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		//Param reading
		Intent act_param = this.getIntent();
		macBTDeviceAddress = act_param.getStringExtra(MSG_MAC_BT_DEVICE_ADDRESS);
		serv_UUID = UUID.fromString(act_param.getStringExtra(MSG_BT_UUID));
		initUI();
	}
	
	/**
	 * Avoid restarting activity when the device rotates.
	 */
	@Override
	public void onConfigurationChanged(Configuration newConfig) {
	  super.onConfigurationChanged(newConfig); 
	  initUI();
	}
	
	@Override
	public void onDestroy()
	{		
		if (intent != null)
		{
			stopService(intent);
		}		
		super.onDestroy();
	}
	
	
	

	private void initUI() {
		setContentView(R.layout.fragment_main);
	
		buttonGps = (Button)findViewById(R.id.buttonGPS);
		buttonGps.setOnClickListener(new View.OnClickListener() 
		{
			@Override
			public void onClick(View v) 
			{
								
				if (!startGPS)
				{
					BluetoothAdapter mBluetoothAdapter = BluetoothAdapter.getDefaultAdapter();
					if (!mBluetoothAdapter.isEnabled()) {
					    Intent enableBtIntent = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
					    startActivityForResult(enableBtIntent, REQUEST_ENABLE_BT);
					    
					    Log.e("BTDeviceDiscover", "Bluetooth is DISABLED. Asking the user");
					    return;
					}
					
					final LocationManager manager = (LocationManager) getSystemService( Context.LOCATION_SERVICE );

				    if (manager.isProviderEnabled( LocationManager.GPS_PROVIDER ) ) {
					    			    	
						//Start the Gps Service				
						intent = new Intent(MainActivity.this, GpsService.class);
			    	    intent.putExtra(GpsService.MSG_BT_UUID, MainActivity.serv_UUID.toString());
			    	    intent.putExtra(GpsService.MSG_MAC_BT_DEVICE_ADDRESS, macBTDeviceAddress);
			    	    startService(intent);
			    	    buttonGps.setText("Stop GPS");
						startGPS = true;
				    }
				    else
				    {
				    	showMsg("GPS is disabled");
				    }
				}
				else
				{
					
					if (intent != null)
					{
						stopService(intent);
					}
					buttonGps.setText("Start GPS");
					startGPS = false;
				}
			}
		});		
	}

	private void showMsg(final String msg) {
		runOnUiThread(new Runnable() {
			@Override
			public void run() {
				Toast.makeText(MainActivity.this, msg, TOAST_DURATION).show();
			}
		});
	}
	
}
