/******************************************************************************

    Copyright (c) EUROGICIEL 2014. All rights reserved.
    This source code is subject to the terms of the ORA SDK License Agreement
    nevertheless, as an exception to this agreement, EUROGICIEL grants you the
    rights to publish, distribute, translate or reproduce it under your own
    responsibility.
    Please visit the following for more information :
    http://www.eurogiciel.fr/index.php/en/products/ora-s-sdk/license

    EUROGICIEL
    417 L'Occitane - CS 77679, 31676 LABEGE Labège Cedex  - France
    Phone: +33 (0)5 61 00 79 79     
    http://www.eurogiciel.fr

******************************************************************************/

package fr.eurogiciel.ora.btclientsample;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.UUID;

import fr.eurogiciel.ora.btclientsample.R;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.Fragment;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.os.ParcelUuid;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.ArrayAdapter;
import android.widget.ListView;

public class ConnectionActivity extends Activity {
	
	private static final String TAG = ConnectionActivity.class.getName();
	private static final int REQUEST_ENABLE_BT = 1;
	
	private static final UUID serv_UUID = UUID.fromString("00001101-0000-1000-8000-00805F9B34AC");
	List<BluetoothDevice> validDevices = new ArrayList<>();
	
	ListView devListView;
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_login);

		if (savedInstanceState == null) {
			getFragmentManager().beginTransaction()
					.add(R.id.container, new PlaceholderFragment()).commit();
		}
	}

	@Override
	protected void onStart() {
		super.onStart();
		devListView = (ListView)findViewById(R.id.server_listview);
		
		//Create a broadcastreceiver that is trigerred asynchronously after a call to fetchUuidsWithSdp() call.
		BroadcastReceiver handler = new BroadcastReceiver() 
		{
			@Override
			public void onReceive(Context context, Intent intent) {
				
				//Check that we just received an UUID discover message
				if (intent.getAction().equals(BluetoothDevice.ACTION_UUID)) {
					
					Bundle bdl = intent.getExtras();
					BluetoothDevice btDevice = (BluetoothDevice) bdl
							.getParcelable(BluetoothDevice.EXTRA_DEVICE);

					Log.e(TAG, "ACTION_UUID event received. Searching through available UUID on " 
							+ btDevice.getAddress());
					
					ParcelUuid[] puList = btDevice.getUuids();

					// Search our UUID server through the list of available UUID
					// on the remote device
					for (ParcelUuid pu : puList) {
						if (pu.getUuid().compareTo(serv_UUID) == 0) {
							Log.e(TAG, "Compatible server found.");
							
							//Check if the device is not already present in the list. 
							if (validDevices.indexOf(btDevice) == -1)
								validDevices.add(btDevice);
							
							break;
						}
					}
					
					//Refresh the GUI
					if (validDevices.size() == 0) {
						Log.e(TAG, "No compatible devices found.");
						devListView.setClickable(false);
						String[] noDeviceFound = {"Please pair your device with ORA Glasses before using ORA connect"};
						devListView.setAdapter(new ArrayAdapter<String>(ConnectionActivity.this,  
								android.R.layout.simple_list_item_1, noDeviceFound));
						
						devListView.setOnItemClickListener(null);
					}
					else
					{
						//Refresh the list of devices on the GUI.
						BTListAdapter adapter = new BTListAdapter(ConnectionActivity.this, validDevices);
						devListView.setAdapter(adapter);
						devListView.setClickable(true);
						
						OnItemClickListener clickHandler = new OnItemClickListener() {
							@Override
							public void onItemClick(AdapterView<?> parent, View view,
									int position, long id) {
								
								BluetoothDevice btDevice = validDevices.get(position);
								
								//Start the main activity
					    		Intent intent = new Intent(ConnectionActivity.this, RemoteControlActivity.class);
					    	    intent.putExtra(RemoteControlActivity.MSG_BT_UUID, ConnectionActivity.serv_UUID.toString());
					    	    intent.putExtra(RemoteControlActivity.MSG_MAC_BT_DEVICE_ADDRESS, btDevice.getAddress());
					    	    
					    		startActivity(intent);
							}
							
						};
						
						//Register the event on click.
						devListView.setOnItemClickListener(clickHandler);
					}
				}
			}
		};
		
		IntentFilter filter = new IntentFilter(BluetoothDevice.ACTION_UUID);
		
		registerReceiver(handler, filter);
		
		//Refresh the listview of MAC addresses and names.
		RefreshDeviceList();
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {

		// Inflate the menu; this adds items to the action bar if it is present.
		getMenuInflater().inflate(R.menu.login, menu);
		return true;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		// Handle action bar item clicks here. The action bar will
		// automatically handle clicks on the Home/Up button, so long
		// as you specify a parent activity in AndroidManifest.xml.
		int id = item.getItemId();
		if (id == R.id.action_settings) {
			return true;
		}
		return super.onOptionsItemSelected(item);
	}

	/**
	 * A placeholder fragment containing a simple view.
	 */
	public static class PlaceholderFragment extends Fragment {

		public PlaceholderFragment() {
		}

		@Override
		public View onCreateView(LayoutInflater inflater, ViewGroup container,
				Bundle savedInstanceState) {
			View rootView = inflater.inflate(R.layout.fragment_login,
					container, false);
			return rootView;
		}
	}
	
	/**
	 * Callback for the refresh button.
	 * @param v
	 */
	public void onBtnClicked(View v){
	    if(v.getId() == R.id.btn_refresh){
	        RefreshDeviceList();
	    }
	}
	
	/**
	 * Refresh the list of paired devices. Note that a device should be paired to be listed
	 * in the listview on this GUI.
	 */
	private void RefreshDeviceList() {
		BluetoothAdapter mBluetoothAdapter = BluetoothAdapter.getDefaultAdapter();
		
		//Flush the list of valid devices
		validDevices.clear();
		
		if (mBluetoothAdapter == null) 
			Log.e("BTDeviceDiscover", "No bluetooth adapter detected/available.");
		
		//Get the list of paired devices.
		Set<BluetoothDevice> devList = mBluetoothAdapter.getBondedDevices();
		
		//Check if bluetooth is enabled.
		if (!mBluetoothAdapter.isEnabled()) {
		    Intent enableBtIntent = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
		    startActivityForResult(enableBtIntent, REQUEST_ENABLE_BT);
		    
		    Log.e("BTDeviceDiscover", "Bluetooth is DISABLED. Asking the user");
		    return;
		}
		
		for (BluetoothDevice btDevice : devList) {
			//Refresh the list of UUID on the remote device. !!!!ASYNCHRONE
			Log.e(TAG, "READING FROM " + btDevice.getAddress());
			btDevice.fetchUuidsWithSdp();
		}
	}
	
	
	
	/**
	 * Callback for the dialog that ask the user to activate the bluetooth.
	 * This method will automatically close the application if the user didn't activated Bluetooth.
	 * @param requestCode Code that identify to which code the callback is returning. Should be equal to
	 * REQUEST_ENABLE_BT.
	 * @param resultCode Result of the user action.
	 * @param data Not used.
	 */
	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {

		//Check the result of the Bluetooth activation request.
	    if (requestCode == REQUEST_ENABLE_BT) {
	        if(resultCode == RESULT_OK){
	            RefreshDeviceList();
	        }
	        if (resultCode == RESULT_CANCELED) {
	        	//If the BLuetooth is till not activated, inform the user that the application will be closed.
	            AlertDialog.Builder builder = new AlertDialog.Builder(this);
	            builder.setTitle("Application will close...")
	            	   .setMessage("This application can't work without Bluetooth.")
	            	   .setCancelable(false)
	            	   .setPositiveButton("OK", new DialogInterface.OnClickListener() {
						
						@Override
						public void onClick(DialogInterface dialog, int which) {
							finish();
						}
					});
	            AlertDialog dialog = builder.create();
	            dialog.show();
	        }
	    }
	    

	}
}
