/******************************************************************************

    Copyright (c) EUROGICIEL 2014. All rights reserved.
    This source code is subject to the terms of the ORA SDK License Agreement
    nevertheless, as an exception to this agreement, EUROGICIEL grants you the
    rights to publish, distribute, translate or reproduce it under your own
    responsibility.
    Please visit the following for more information :
    http://www.eurogiciel.fr/index.php/en/products/ora-s-sdk/license

    EUROGICIEL
    417 L'Occitane - CS 77679, 31676 LABEGE Labège Cedex  - France
    Phone: +33 (0)5 61 00 79 79     
    http://www.eurogiciel.fr

******************************************************************************/

package fr.eurogiciel.ora.btclientsample;


import java.io.IOException;
import java.util.UUID;

import android.annotation.SuppressLint;
import android.app.Activity;
import android.app.ProgressDialog;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.content.Context;
import android.content.Intent;
import android.content.res.Configuration;
import android.gesture.GestureOverlayView;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.util.Log;
import android.view.KeyEvent;
import android.view.MotionEvent;
import android.view.View;
import android.view.View.OnTouchListener;
import android.view.inputmethod.InputMethodManager;
import android.widget.Button;
import android.widget.Toast;
import fr.eurogiciel.ora.bluetooth.BTClient;
import fr.eurogiciel.ora.bluetooth.BTMessage;
import fr.eurogiciel.ora.bluetooth.BTSocketListener;
import fr.eurogiciel.ora.bluetooth.event.EventMsgFactory;
import fr.eurogiciel.ora.bluetooth.gps.GpsMsgFactory;

/**
 * This sample code demonstrates how to transform an android device in a bluetooth mouse and a bluetooth keyboard.
 * It assumes another (bluetooth accessible) android device running a {@link PServer}.
 * To drag objects, the user must use two fingers.
 */
public class RemoteControlActivity extends Activity implements BTSocketListener.Callback, OnTouchListener {
	
	private static final int TOAST_DURATION = 2000;
	private final String TAG = getClass().getPackage().getName();
	public final static String MSG_BT_UUID = "MSG_BT_UUID";
	public final static String MSG_MAC_BT_DEVICE_ADDRESS = "MSG_MAC_BT_DEVICE_ADDRESS";
	
	private boolean connected = false;

	private BluetoothDevice device;
	
	//IHM
	private	GestureOverlayView	gestureView = null;
	private BTClient client;
	private InputMethodManager	virtkeyboard = null;
	private Button keyboard = null;
	private ProgressDialog mProgressDialog;
	
	//GPS
	private static final long GPS_MIN_TIME = 250; //minimum time interval between location updates, in milliseconds 
	private static final float GPS_MIN_DISTANCE = 0.0f;	//minimum distance between location updates, in meters
	private static final long CLICK_TIME_THREESHOL = 100; //under this duration, a touch is a click, else it is a move, in milliseconds
	private boolean enabledGPS;
	private EventMsgFactory eventMsgFactory = new EventMsgFactory();
	private GpsMsgFactory	gpsMsgFactory	= new GpsMsgFactory();
	private MyLocationListener locationListener;

	//mouse control
	private float lastX = -1;
	private float lastY = -1;
	private long startTouchTime = -1;
	private BTMessage downMsg = eventMsgFactory.createMouseDownEventMsg();
	private BTMessage upMsg = eventMsgFactory.createMouseUpEventMsg();
	private boolean dragging;
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		
		//Param reading
		Intent act_param = this.getIntent();
		final String macBTDeviceAddress = act_param.getStringExtra(MSG_MAC_BT_DEVICE_ADDRESS);
		final UUID serv_UUID = UUID.fromString(act_param.getStringExtra(MSG_BT_UUID));
		
		enabledGPS = enableLocationMsgs();
		
		BluetoothAdapter ba = BluetoothAdapter.getDefaultAdapter();
		
		//retrieve the bt device
		for (BluetoothDevice d : ba.getBondedDevices()) {
			if (d.getAddress().equals(macBTDeviceAddress)) {
				device = d;
				break;
			}
		}
		
		if (device == null) {
			throw new IllegalArgumentException();
		}
		
		//Progress dialog
		mProgressDialog = ProgressDialog.show(this, "Please wait",
	            "Trying to connect to \"" + device.getName() + "\"" , true);
		
		//Connection thread
		new Thread("Client creation") {
			public void run() {
				try {
					client = new BTClient(RemoteControlActivity.this, RemoteControlActivity.this, device, serv_UUID);
					
					//connection successfull
					connected = true;
			        mProgressDialog.dismiss();
			        showMsg("Connection successful!");
				
				} catch (IOException e) {
					showMsg(e.getMessage());
					Log.e(TAG, e.getMessage(), e);
					finish();
				}
			};
		}.start();
		
		initUI();
	}
	
	/**
	 * Avoid restarting activity when the device rotates.
	 */
	@Override
	public void onConfigurationChanged(Configuration newConfig) {
	  super.onConfigurationChanged(newConfig); 
	  initUI();
	}
	
	@Override
	public void onDestroy()
	{
		stopGPSMsgs();
		
		if (client != null) {
			client.close();
		}
		
		if (mProgressDialog != null) {
			mProgressDialog.dismiss();
		}
		
		super.onDestroy();
	}
	
	/**
	 * Should never receive messages in this application.
	 */
	@Override
	public BTMessage onMsgReceived(BluetoothDevice sender, BTMessage msg) {
		return null;
	}

	/**
	 * Authorization always granted.
	 */
	@Override
	public boolean isAuthorized(BluetoothDevice client) {
		return true;
	}
	
	@Override
	public void onDisconnected(BluetoothDevice server) {
		stopGPSMsgs();
        showMsg("Disconnected from server.");
		finish();
	}

	/**
	 * Sends the key event to the remote bluetooth device.
	 */
	@Override
	public boolean dispatchKeyEvent(KeyEvent event)
	{
		if (connected && event.getAction() == KeyEvent.ACTION_DOWN) {
			try {
				client.sendMessage(eventMsgFactory.createKeyEventMsg(event));
			} catch (IOException e) {
				showMsg(e.getMessage());
				Log.e(TAG, "error", e);
			}
		}
		
	    return super.dispatchKeyEvent(event);
	}

	@SuppressLint("ClickableViewAccessibility") //the server performs the click
	@Override
	/**
	 * Create and send to the server mouse events from touch events.
	 * One finger to move the cursor, two to drag elements.
	 */
	public boolean onTouch(View arg0, MotionEvent motionEvent) {
		
		if (connected) {
			try {
				
				int pointerCount = motionEvent.getPointerCount();
				int action = motionEvent.getAction();
				boolean moving = startTouchTime != -1 && System.currentTimeMillis() - startTouchTime > CLICK_TIME_THREESHOL;
				
				if (action == MotionEvent.ACTION_DOWN) {
					startTouchTime = System.currentTimeMillis();
				}
				
				if (action == MotionEvent.ACTION_UP) {
					if (! moving) {
						//perform click on server
						client.sendMessage(downMsg);
						client.sendMessage(upMsg);
					}
					
					startTouchTime = -1;
				}
				
				
				if (pointerCount >= 2 && !dragging && moving) {
					//start dragging (at least two fingers)
					client.sendMessage(downMsg);
					dragging = true;
				}
				
				else if (pointerCount < 2 && dragging) {
					//stop dragging
					client.sendMessage(upMsg);
					dragging = false;
				}
				
				else if (action == MotionEvent.ACTION_MOVE && lastX != -1 && lastY != -1 && moving) {
					//move cursor
					int dx = (int) (motionEvent.getX() - lastX);
					int dy = (int) (motionEvent.getY() - lastY);
 					BTMessage moveMsg = eventMsgFactory.createMouseMoveEventMsg(dx, dy);
 					client.sendMessage(moveMsg);
				}
				
				lastX = motionEvent.getX();
				lastY = motionEvent.getY();
				
				return true;
			} catch (IOException e) {
				e.printStackTrace();
				Toast.makeText(this, "Sending touch event failed.", TOAST_DURATION).show();
			}
		}
		return false;
	}

	private void initUI() {
		setContentView(R.layout.fragment_main);
	
		keyboard     = (Button)findViewById(R.id.btKeyboard);
		gestureView = (GestureOverlayView)findViewById(R.id.gestureOverlayView1);
		gestureView.setOnTouchListener(this);
		virtkeyboard = (InputMethodManager)getSystemService(Context.INPUT_METHOD_SERVICE);
		
		keyboard.setOnClickListener(new View.OnClickListener() 
		{
			@Override
			public void onClick(View v) 
			{
				virtkeyboard.toggleSoftInput(InputMethodManager.SHOW_IMPLICIT, InputMethodManager.HIDE_IMPLICIT_ONLY);
			}
		});
	}

	private boolean enableLocationMsgs() {
		
		try {
			
			LocationManager locationManager = (LocationManager)getSystemService(Context.LOCATION_SERVICE);	
			locationListener = new MyLocationListener();
			locationManager.requestLocationUpdates(LocationManager.GPS_PROVIDER, GPS_MIN_TIME, GPS_MIN_DISTANCE, locationListener);
			
			return true;
		} 
		
		//GPS feature is not available
		catch (SecurityException e) {} 
		catch (NullPointerException e) {}
		catch (IllegalArgumentException e) {}
		return false;
	}

	private void stopGPSMsgs() {
		if (enabledGPS) {
			LocationManager locationManager = (LocationManager)getSystemService(Context.LOCATION_SERVICE);
			locationManager.removeUpdates(locationListener);
			locationListener = null;
			enabledGPS = false;
		}
	}
	
	private void showMsg(final String msg) {
		runOnUiThread(new Runnable() {
			@Override
			public void run() {
				Toast.makeText(RemoteControlActivity.this, msg, TOAST_DURATION).show();
			}
		});
	}

	private class MyLocationListener implements LocationListener 
	{
		@Override
		public void onLocationChanged(Location loc) {
	
			if (client != null && loc != null)
			{
				try {
					client.sendMessage(gpsMsgFactory.createGpsMsg(loc));
				} catch (IOException e) {
					showMsg(e.getMessage());
					Log.e(TAG, "error", e);
				}
			}
		}
	
		@Override
		public void onProviderDisabled(String provider) {
			stopGPSMsgs();
		}
	
		@Override
		public void onProviderEnabled(String provider) {}
	
		@Override
		public void onStatusChanged(String provider, int status, Bundle extras) {}
	}
	
}
