/******************************************************************************

    Copyright (c) EUROGICIEL 2014. All rights reserved.
    This source code is subject to the terms of the ORA SDK License Agreement
    nevertheless, as an exception to this agreement, EUROGICIEL grants you the
    rights to publish, distribute, translate or reproduce it under your own
    responsibility.
    Please visit the following for more information :
    http://www.eurogiciel.fr/index.php/en/products/ora-s-sdk/license

    EUROGICIEL
    417 L'Occitane - CS 77679, 31676 LABEGE Labège Cedex  - France
    Phone: +33 (0)5 61 00 79 79     
    http://www.eurogiciel.fr

******************************************************************************/

package fr.eurogiciel.ora.btserversample;

import java.io.IOException;

import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothSocket;
import android.location.LocationManager;
import android.util.Log;

import fr.eurogiciel.ora.bluetooth.BTMessage;
import fr.eurogiciel.ora.bluetooth.BTServerService;
import fr.eurogiciel.ora.bluetooth.event.Event;
import fr.eurogiciel.ora.bluetooth.event.EventInjector;
import fr.eurogiciel.ora.bluetooth.gps.GPS;
import fr.eurogiciel.ora.bluetooth.gps.MockLocationProvider;

/**
 * This sample code shows how to create a service that allows to use an external bluetooth device as a remote keyboard or a mouse.
 * It uses the {@link BTServerService} and the {@link EventInjector} features to do so.
 */
public class EventInjectorService extends BTServerService {

	private static final String TAG = EventInjectorService.class.getPackage().getName();
    
    private EventInjector eventInjector;
	private MockLocationProvider mockLocationProvider; 

    @Override
    public void onCreate() {
    	mockLocationProvider = new MockLocationProvider(LocationManager.GPS_PROVIDER, this);
    	try {
			eventInjector = new EventInjector();
		} catch (IOException e) {
			Log.e(TAG, "Error while initializing event file.", e);
			stopSelf();
		}
    	super.onCreate();
    }

	@Override
	public boolean isAuthorized(BluetoothDevice client) {
		//TODO : add auth system
		return true;
	}

	@Override
	public BTMessage onMsgReceived(BluetoothDevice client, BTMessage msg) {
		
		switch (msg.getMsgType()) {
		case GPS.GPS_MSG_TYPE:
			return mockLocationProvider.handle(msg);
		case Event.MSG_TYPE_KEYBOARD:
		case Event.MSG_TYPE_MOUSE_DOWN:
		case Event.MSG_TYPE_MOUSE_UP:
		case Event.MSG_TYPE_MOUSE_MOVE:
			return eventInjector.handle(msg);
		default:
			Log.w(TAG, "Unsupported msg type, ignoring.");
			return null;
		}
	}

	@Override
	public void onServerStopped() {
		Log.i(TAG, "server stopped");
	}
	
	@Override
	public void onDisconnected(BluetoothDevice device) {
		Log.i(TAG, device.getName() + " disconnected");
	}

	@Override
	public void onClientConnected(BluetoothSocket socket) {
		//nothing to do
	}
    
}
